#ifndef CURVATURE_ORACLE_H
#define CURVATURE_ORACLE_H

#include <pthread.h>
#include <math.h>

#include "neighbours_list.hpp"
#include "proximity_verifier.hpp"
#include "transform3d.hpp"
#include "k2tree.hpp"
#include "config.hpp"
#include "point_record.hpp"
#include "trianglemesh.hpp"
#include "neighbours_cache.hpp"
#include "stack_int.hpp"
#include "math_utils.hpp"
#include "vector.hpp"
#include "sampler.hpp"

#include <vector>

typedef struct {
	int start;
	int n;

	PointRecord* points;
	int points_size;

	const NeighboursCache* cache;
	const TriangleMesh* mesh;
	int* samples;
	NeighboursList* list;

} CurvatureOraclePRArgs;

typedef struct {
	Transform3D* candidates;
	int size;
	int capacity;
} CandidatesTest;

typedef struct {
	int start;
	int n;

	int candidates_size;

	CandidatesTest* ct;
	//std::vector<Transform3D>* candidates;

	K2Tree* tree;
	const ProximityVerifier* verifier;

	const NeighboursCache* cache;
	const TriangleMesh* mesh;
} CurvatureOracleFMArgs;

/* Finds candidate transformations by computing curvatures
 * of both sampled meshes and then mapping points with similar
 * curvatures to each other */
Transform3D* curvature_oracle_find_candidates(TriangleMesh* mesh1, TriangleMesh* mesh2, ProximityVerifier* verifier, int* candidates_size);

#endif
