#ifndef DISTANCE_TREE_H
#define DISTANCE_TREE_H

#include <stdlib.h>
#include "transform3d.hpp"
#include "metric.hpp"
#include "vector_int.hpp"
#include "vector.hpp"
#include "point3d.hpp"
#include <vector>

#define DISTANCE_TREE_LEAF_CHILDREN_COUNT 5

typedef enum {
	LEAF, NON_LEAF
} DistanceTreeNodeType;

typedef struct _DistanceTreeNode {
	/* Type of node - basically if node is a leaf or not. */
	DistanceTreeNodeType type;

	/* Index of candidate which the node represents */
	int candidate_index;

	struct _DistanceTreeNode* close;
	struct _DistanceTreeNode* far;

	/* In case node is a leaf array of indexes of candidates belonging to node */
	/* TODO: Potential optimalization */
	int* children;
	int children_size;

	/* Value determining if node/candidate belongs to 'close' or 'far' child */
	float threshold;

} DistanceTreeNode;

typedef struct {
	DistanceTreeNode* root;
	Transform3D* candidates;
	int candidates_size;
} DistanceTree;

DistanceTree* distance_tree_create(Transform3D* candidates, int candidates_size);

std::vector<int> distance_tree_find_close(DistanceTree* tree, Transform3D candidate, float threshold);
void distance_tree_free(DistanceTree* tree);

#endif
