#ifndef LOGGER_H
#define LOGGER_H

#include <stdio.h>

//#define LOG(S) printf("#   %s:%d   # %s\n", __FILE__, __LINE__, S);
//#define LOG_FORMATTED(...) printf("#   %s:%d   # ", __FILE__, __LINE__); printf(__VA_ARGS__);
#ifdef DEBUG
	#define LOG(S) printf("%s\n", S);
	#define LOG_FORMATTED(...) printf(__VA_ARGS__);
	#define LOG_DO_IF_LOGGER_ENABLED(F) F;
#else
	#define LOG(S)
	#define LOG_FORMATTED(...)
	#define LOG_DO_IF_LOGGER_ENABLED(F)
#endif

#ifdef PROFILING
	#ifndef _GLIBCXX_CHRONO
		#include <chrono>
	#endif

	#ifndef _GLIBCXX_IOSTREAM
		#include <iostream>
	#endif

	#define PROFILER_START_TIMER(VAR) auto VAR = std::chrono::high_resolution_clock::now();
	#define LOG_TIME_SPENT_SINCE(X, S) std::cout << S << std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::high_resolution_clock::now() - X).count() << "ms" << std::endl;
#else
	#define PROFILER_START_TIMER(VAR) 
	#define LOG_TIME_SPENT_SINCE(X, S) 
#endif


#endif
