#ifndef POINT3D_H
#define POINT3D_H

#include <math.h>
#include <immintrin.h>

class Point3D {
public:
	union {
		struct {
			float x, y, z, _padding;
		};
		__m128 vector;
		float coords[4];
	};

	Point3D() {
	}

	Point3D(float x, float y, float z) {
		this->x = x;
		this->y = y;
		this->z = z;
		this->_padding = 0;
	}

	Point3D(__m128 vector) {
		this->vector = vector;
	}

	Point3D operator+(const Point3D& p) {
		return Point3D(_mm_add_ps(this->vector, p.vector));
	}

	void operator+=(const Point3D& p) {
		this->vector = _mm_add_ps(this->vector, p.vector);
	}

	Point3D operator-(const Point3D& p) {
		return Point3D(_mm_sub_ps(this->vector, p.vector));
	}

	void operator-=(const Point3D& p) {
		this->vector = _mm_sub_ps(this->vector, p.vector);
	}

	Point3D operator*(float s) {
		Point3D res;

		res.x = this->x * s;
		res.y = this->y * s;
		res.z = this->z * s;

		return res;
	}

	void operator*=(float s) {
		this->x *= s;
		this->y *= s; 
		this->z *= s;
	}

	float abs() {
		return sqrt(this->x * this->x + this->y * this->y + this->z * this->z);
	}

	void negate() {
		this->x = -this->x;
		this->y = -this->y;
		this->z = -this->z;
	}

	static float distance(Point3D p1, Point3D p2) {
		return sqrt((p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z));
	}

	static float distance_squared(Point3D p1, Point3D p2) {
		return (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z);
	}

	static float dotproduct(Point3D p1, Point3D p2) {
		return p1.x * p2.x + p1.y * p2.y + p1.z * p2.z;
	}

	static Point3D crossproduct(Point3D p1, Point3D p2) {
		Point3D res;

		res.x = p1.y * p2.z - p2.y * p1.z; 
		res.y = p1.z * p2.x - p2.z * p1.x;
		res.z = p1.x * p2.y - p2.x * p1.y;

		return res;
	}

};


//typedef union {
//	struct {
//		float x, y, z, _padding;
//	};
//	__m128 vector;
//} Point3D;
//
//inline Point3D create_point3d(float x, float y, float z) {
//	Point3D point;
//
//	point.x = x;
//	point.y = y;
//	point.z = z;
//
//	return point;
//}

//inline void point_negative_ref(Point3D* p1) {
//	p1->x = -p1->x;
//	p1->y = -p1->y;
//	p1->z = -p1->z;
//}
//
//inline Point3D POINTS_ADD(Point3D p1, Point3D p2) {
//	Point3D res;
//
//	res.vector = _mm_add_ps(p1.vector, p2.vector);
//
//	return res;
//}
//
//inline void POINTS_SUB_REF(Point3D* p1, Point3D p2) {
//	p1->vector = _mm_sub_ps(p1->vector, p2.vector);
//}
//
//inline Point3D POINTS_SUB(Point3D p1, Point3D p2) {
//	Point3D res;
//
//	res.vector = _mm_sub_ps(p1.vector, p2.vector);
//
//	return res;
//}
//
//inline Point3D POINTS_CROSSPRODUCT(Point3D p1, Point3D p2) {
//	Point3D res;
//
//	res.x = p1.y * p2.z - p2.y * p1.z; 
//	res.y = p1.z * p2.x - p2.z * p1.x;
//	res.z = p1.x * p2.y - p2.x * p1.y;
//
//	return res;
//}
//
//inline float POINTS_DOTPRODUCT(Point3D p1, Point3D p2) {
//	return p1.x * p2.x + p1.y * p2.y + p1.z * p2.z;
//}
//
//inline float POINT_ABS(Point3D p) {
//	return sqrt(p.x * p.x + p.y * p.y + p.z * p.z);
//}
//
//inline Point3D POINT_SCALE(Point3D p, float s) {
//	Point3D res;
//
//	res.x = p.x * s;
//	res.y = p.y * s;
//	res.z = p.z * s;
//
//	return res;
//}
//
//inline void POINT_SCALE_REF(Point3D* p, float s) {
//	p->x *= s;
//	p->y *= s; 
//	p->z *= s;
//}
//
//inline void POINTS_ADD_REF(Point3D* p1, Point3D p2) {
//	p1->vector = _mm_add_ps(p1->vector, p2.vector);
//}
//
//inline float POINTS_DISTANCE(Point3D p1, Point3D p2) {
//	return sqrt((p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z));
//}
//
//inline float points_distance_squared(Point3D p1, Point3D p2) {
//	return (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z);
//}
//
//
//void print_point3d(Point3D p);

#endif
