#ifndef TRANSFORM3D_H
#define TRANSFORM3D_H

#include <string.h>
#include <stdio.h>

#include <pmmintrin.h>
#include <immintrin.h>

#include "point_record.hpp"
#include "point3d.hpp"
#include "matrix3x3.hpp"

class Transform3D {
public:

	float rotation[9];
	union {
		float translation[4];
		__m128 translation_vector;
	};

	Transform3D() {
	}

	Transform3D(float* rotation, float* translation) {
		memcpy(this->rotation, rotation, sizeof(float) * 9);
		memcpy(this->translation, translation, sizeof(float) * 4);
	}

	Transform3D operator*(const Transform3D& t) {
		Transform3D result;
		memset(result.rotation, '\0', sizeof(float) * 9);

		for(int col = 0; col < 3; ++col) {
			for(int row = 0; row < 3; ++row) {
				for(int i = 0; i < 3; ++i) {
					result.rotation[col * 3 + row] += this->rotation[col * 3 + i] * t.rotation[row + 3 * i];
				}
			}
		}

		for(int i = 0; i < 3; ++i) {
			result.translation[i] = this->rotation[i * 3] * t.translation[0];
			result.translation[i] += this->rotation[i * 3 + 1] * t.translation[1];
			result.translation[i] += this->rotation[i * 3 + 2] * t.translation[2];
			result.translation[i] += this->translation[i];
		}

		return result;
	};

	Point3D operator*(const Point3D& p) {
		Point3D result;

		__m128 col0 = _mm_set_ps(0, rotation[6], rotation[3], rotation[0]);
		__m128 colp0 = _mm_set_ps1(p.x);
		__m128 res0 = _mm_mul_ps(col0, colp0);

		__m128 col1 = _mm_set_ps(0, rotation[7], rotation[4], rotation[1]);
		__m128 colp1 = _mm_set_ps1(p.y);
		__m128 res1 = _mm_mul_ps(col1, colp1);

		__m128 col2 = _mm_set_ps(0, rotation[8], rotation[5], rotation[2]);
		__m128 colp2 = _mm_set_ps1(p.z);
		__m128 res2 = _mm_mul_ps(col2, colp2);

		__m128 add01 = _mm_add_ps(res0, res1);
		__m128 add012 = _mm_add_ps(res2, add01);

		result.vector = _mm_add_ps(add012, translation_vector);

		return result;
	};

	void transform_point_ref(Point3D& p) {
		__m128 col0 = _mm_set_ps(0, rotation[6], rotation[3], rotation[0]);
		__m128 colp0 = _mm_set_ps1(p.x);
		__m128 res0 = _mm_mul_ps(col0, colp0);

		__m128 col1 = _mm_set_ps(0, rotation[7], rotation[4], rotation[1]);
		__m128 colp1 = _mm_set_ps1(p.y);
		__m128 res1 = _mm_mul_ps(col1, colp1);

		__m128 col2 = _mm_set_ps(0, rotation[8], rotation[5], rotation[2]);
		__m128 colp2 = _mm_set_ps1(p.z);
		__m128 res2 = _mm_mul_ps(col2, colp2);

		p.vector = _mm_add_ps(translation_vector, _mm_add_ps(res2, _mm_add_ps(res0, res1)));
	}

	void rotate_point_ref(Point3D& p) {
		__m128 col0 = _mm_set_ps(0, rotation[6], rotation[3], rotation[0]);
		__m128 colp0 = _mm_set_ps(0, p.x, p.x, p.x);
		__m128 res0 = _mm_mul_ps(col0, colp0);

		__m128 col1 = _mm_set_ps(0, rotation[7], rotation[4], rotation[1]);
		__m128 colp1 = _mm_set_ps(0, p.y, p.y, p.y);
		__m128 res1 = _mm_mul_ps(col1, colp1);

		__m128 col2 = _mm_set_ps(0, rotation[8], rotation[5], rotation[2]);
		__m128 colp2 = _mm_set_ps(0, p.z, p.z, p.z);
		__m128 res2 = _mm_mul_ps(col2, colp2);

		p.vector = _mm_add_ps(res2, _mm_add_ps(res0, res1));
	}
};

void transform3d_print(Transform3D t);
Transform3D transform3d_create_map(PointRecord source, PointRecord target, int swap);

#endif
