#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include "obj_loader.hpp"
#include "trianglemesh.hpp"
#include "triangle.hpp"
#include "point3d.hpp"
#include "transform3d.hpp"
#include "math_utils.hpp"
#include "cluster_registration.hpp"
#include "config.hpp"

#include "vector.hpp"
#include "time_measurement.hpp"
#include <chrono>
#include "logger.hpp"

float config_delta_ratio = 0.02;
float config_sample_size_mesh1 = 2000;
float config_sample_size_mesh2 = 400;
float config_curvatures_calculated = 15000;
float config_candidates_length = 10000;
float config_neighbourhood_width = 8;
float config_rotation_coeff = 9.52;
float config_acceptace_threshold = 0.03;
float config_curvature_ratio = 0.7;
int config_number_of_processors;

/* TODO: 
 * 	- magic values in curvature_oracle
 * 	- delete useless files
 * 	- swap vector.h with std::vector
 * 	- nonmanifold meshes
 * 	- obj loader to FSM
 * 	- comments(especially in header files)
 * */
int main(int argc, char** argv) {

	PROFILER_START_TIMER(PROGRAM_EXECUTION)

	/* Setting number of threads, either via sys call in Linux or
	 * just setting the constant 
	 * TODO: Add option to set this via argv */
#ifdef __linux__
	config_number_of_processors = sysconf(_SC_NPROCESSORS_ONLN);
#else
	config_number_of_processors = 8;
#endif

	srand(time(0));

	LOG("Loading meshes...");
	PROFILER_START_TIMER(LOADING_MESHES);

	LOG("Loading mesh #1...");
	TriangleMesh* mesh1 = get_trianglemesh_from_file(argv[1]);
	LOG("Loading mesh #2...");
	TriangleMesh* mesh2 = get_trianglemesh_from_file(argv[2]);

	if(mesh1 == NULL || mesh2 == NULL) {
		printf("Couldn't parse meshes. Exiting...\n");
		return EXIT_FAILURE;
	}

	LOG("Loading finished succesfully");
	LOG_TIME_SPENT_SINCE(LOADING_MESHES, "Meshes loaded in: ");

	PROFILER_START_TIMER(MESHES_INIT);

	LOG("Initializing mesh #1...");
	init_trianglemesh(mesh1);
	LOG_FORMATTED("Mesh1: points_size: %d\n", mesh1->points_size);

	LOG("Initializing mesh #2...");
	init_trianglemesh(mesh2);
	LOG_FORMATTED("Mesh2: points_size: %d\n", mesh2->points_size);

	LOG_TIME_SPENT_SINCE(MESHES_INIT, "Meshes initialized in: ");

	Transform3D transform = cluster_registration_execute(mesh1, mesh2);
	LOG("Best scoring transformation: ");
	transform3d_print(transform);

	trianglemesh_free(mesh1);
	trianglemesh_free(mesh2);

	LOG_TIME_SPENT_SINCE(PROGRAM_EXECUTION, "Program finished in: ");

	return EXIT_SUCCESS;
}
