
#include <stdlib.h>
#include <mutex>

#include "neighbours_cache.hpp"

NeighboursCache* neighbours_cache_create(int size, CornerTable* ct) {
	NeighboursCache* cache = (NeighboursCache*) malloc(sizeof(NeighboursCache));

	cache->size = size;
	cache->entries = (CacheEntry*) calloc(sizeof(CacheEntry), size);
	cache->corner_table = ct;

	return cache;
}

/* Mutex needed to lock cache to prevent memory leaks. */
static std::mutex cache_mutex;

CacheEntry neighbours_cache_get(const NeighboursCache* cache, int index) {

	/* Index hasn't been accessed yet, need to cache it from corner table */
	if(cache->entries[index].data == NULL) {

		int len;
		int* neighbours = corner_table_neighbours(cache->corner_table, index, &len);

		/* Because of the possible memory leak adding new data to cache needs to be locked */
		cache_mutex.lock();

		/* Need to check for NULL again, since after thread wakes up 
		 * cache could already be filled from other thread */
		if(cache->entries[index].data == NULL) {
			cache->entries[index].data = neighbours;
			cache->entries[index].size = len;
		}

		cache_mutex.unlock();
	}

	return cache->entries[index];
}

void neighbours_cache_free(NeighboursCache** cache) {
	if(cache == NULL || *cache == NULL) {
		return;
	}

	for(int i = 0; i < (*cache)->size; ++i) {
		if((*cache)->entries[i].data != NULL) {
			free((*cache)->entries[i].data);
		}
	}

	free((*cache)->entries);
	free(*cache);
}
